# Copyright 2004,2005 Pierre Martineau <pmartino@users.sourceforge.net>
# This file is part of Bibus, a bibliographic database that can
# work together with OpenOffice.org to generate bibliographic indexes.
#
# Bibus is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bibus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bibus; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
from wxPython.wx import *
import BIBbase, BIB, os.path, cPickle

class BibConfig(wxConfig):
	def __init__(self,appname):
		if wxVERSION[:2] >= (2,5):
			wxConfig.__init__(self)
			self.Set(self.Create())
		else:
			wxConfig.__init__(self,appname,style=wxCONFIG_USE_LOCAL_FILE)

	def myGet(self):
		"""We use Set/Get with wxPython >= 2.5
		But it does not work with 2.4
		=> we just use the instance directly"""
		if wxVERSION[:2] >= (2,5):
			return self.Get()
		else:
			return self

	def readConfig(self):
		"""We set the variables in BIB module"""
		current = self.myGet()
		current.SetPath('/')
		if current.HasGroup('general'):
			BIB.FIRST_START = current.ReadBool('/general/firststart')
			BIB.CONFIG_VERSION = current.Read('/general/config_version')
			BIB.WP = current.Read('/general/wp',BIB.WP)	# word processor connection type OOo/mswDoc
		if current.HasGroup('mysql'):
			BIB.SOCKET = current.Read('/mysql/socket',BIB.SOCKET)
			BIB.PORT = current.ReadInt('/mysql/port',BIB.PORT)
			BIB.HOST = current.Read('/mysql/host',BIB.HOST)
			BIB.USER = current.Read('/mysql/user',BIB.USER)
			BIB.PASSWORD = current.Read('/mysql/passwd',BIB.PASSWORD)
			BIB.DB_NAME = current.Read('/mysql/name',BIB.DB_NAME)
		else:
			if not current.HasEntry('/mysql/user'):
				try:
					BIB.USER = getpass.getuser()	# We try to get a default value if not defined.
				except:
					BIB.USER = ''
		if current.HasGroup('sqlite'):
			BIB.SQLiteFile = current.Read('/sqlite/file',BIB.SQLiteFile)
			BIB.SQLiteUSER = current.Read('/sqlite/user',BIB.SQLiteUSER)
		else:
			if not current.HasEntry('/sqlite/user'):
				try:
					BIB.SQLiteUSER = getpass.getuser()	# We try to get a default value if not defined.
				except:
					BIB.SQLiteUSER = ''
		if current.HasGroup('db'):
			BIB.STORE_PASSWD = current.ReadBool('/db/store_passwd',BIB.STORE_PASSWD)
			BIB.DB_STARTUP = current.ReadInt('/db/last_db',BIB.DB_STARTUP)
			BIB.DB_TYPE = current.Read('/db/type',BIB.DB_TYPE)
			BIB.DUPLICATES_TEST = current.ReadBool('/db/duplicates_test',BIB.DUPLICATES_TEST)	# True if we want to test for duplicate entries
			if current.HasEntry('/db/duplicates'): BIB.DUPLICATES = eval(current.Read('/db/duplicates'))	# fields to test
			BIB.DUPLICATES_CASE = current.ReadBool('/db/duplicates_case',BIB.DUPLICATES_CASE)	# True == comparison is Case sensitive
			BIB.DUPLICATES_KEEP_OLD = current.ReadBool('/db/duplicates_keep_old',BIB.DUPLICATES_KEEP_OLD)	# True == we keep the old ref
		if current.HasGroup('display'):
			if current.HasEntry('/display/fields'): BIB.LIST_DISPLAY = eval(current.Read('/display/fields'))	# Fields displayed in reflist wxListCtrl
			BIB.LIST_ORDER = current.Read('/display/order',BIB.LIST_ORDER)				# Display order in reflist
			if current.HasEntry('/display/col_size'): BIB.LIST_COL_SIZE = eval( current.Read('/display/col_size') ) # column sizes (tuple)
			if current.HasEntry('/display/display_format'): BIB.DISPLAY_FORMAT = current.ReadInt('/display/display_format',BIB.DISPLAY_FORMAT)	# select tab in bottom-right main window
			if current.HasEntry('/display/key_color'): BIB.KEY_COLOR = eval('wxColour'+current.Read('/display/key_color'))		# color display of keys (Title, AUthor, etc..) in textCtrl-1
			if current.HasEntry('/display/text_color'): BIB.TEXT_COLOR = eval('wxColour'+current.Read('/display/text_color'))		# color display of text (Title, AUthor, etc..) in textCtrl-1
			if current.HasEntry('/display/ref_editor'): BIB.EDIT = eval( current.Read('/display/ref_editor')	)	# ReferenceEditor
		if current.HasGroup('search'):
			if current.HasEntry('/search/fields'): BIB.BIB_SEARCH_FIELDS= eval(current.Read('/search/fields'))
		if current.HasGroup('window'):
			BIB.WX = current.ReadInt('/window/x',BIB.WX)
			BIB.WY = current.ReadInt('/window/y',BIB.WY)
			BIB.WIDTH = current.ReadInt('/window/width',BIB.WIDTH)
			BIB.HEIGHT = current.ReadInt('/window/height',BIB.HEIGHT)
			BIB.SASH_LIST = current.ReadInt('/window/sash_list',BIB.SASH_LIST)
			BIB.SASH_KEYTREE = current.ReadInt('/window/sash_keytree',BIB.SASH_KEYTREE)
			BIB.REF_Ed_WIDTH = current.ReadInt('/window/ref_width',BIB.REF_Ed_WIDTH)	# RefEditor width
			BIB.REF_Ed_X = current.ReadInt('/window/ref_x',BIB.REF_Ed_X)				# x pos
			BIB.REF_Ed_Y = current.ReadInt('/window/ref_y',BIB.REF_Ed_Y)				# y pos
			BIB.IMPORT_WIDTH = current.ReadInt('/window/import_width',BIB.IMPORT_WIDTH)	# text Import window
			BIB.IMPORT_HEIGHT = current.ReadInt('/window/import_height',BIB.IMPORT_HEIGHT)# text Import width
			BIB.IMPORT_X = current.ReadInt('/window/import_x',BIB.IMPORT_X)				# x pos
			BIB.IMPORT_Y = current.ReadInt('/window/import_y',BIB.IMPORT_Y)				# y pos
			BIB.IMPORT_FORMAT = current.ReadInt('/window/import_format',BIB.IMPORT_FORMAT)	# import format
			BIB.SEARCH_WIDTH = current.ReadInt('/window/search_width',BIB.SEARCH_WIDTH)	# text Import window
			BIB.SEARCH_HEIGHT = current.ReadInt('/window/search_height',BIB.SEARCH_HEIGHT)# text Import width
			BIB.SEARCH_X = current.ReadInt('/window/search_x',BIB.SEARCH_X)				# x pos
			BIB.SEARCH_Y = current.ReadInt('/window/search_y',BIB.SEARCH_Y)				# y pos
			BIB.PUBMED_WIDTH = current.ReadInt('/window/pubmed_width',BIB.PUBMED_WIDTH)	# text Import window
			BIB.PUBMED_HEIGHT = current.ReadInt('/window/pubmed_height',BIB.PUBMED_HEIGHT)# text Import width
			BIB.PUBMED_X = current.ReadInt('/window/pubmed_x',BIB.PUBMED_X)				# x pos
			BIB.PUBMED_Y = current.ReadInt('/window/pubmed_y',BIB.PUBMED_Y)				# y pos
			BIB.PUBMED_NB = current.ReadInt('/window/pubmed_nb',BIB.PUBMED_NB)			# number of pubmed record to import
		if current.HasGroup('OOo'):	# OOo settings
			BIB.OO_CON_TYPE = current.ReadInt('/OOo/con_type',BIB.OO_CON_TYPE)
			BIB.OO_HOST = current.Read('/OOo/host',BIB.OO_HOST)
			BIB.OO_PORT = current.ReadInt('/OOo/port',BIB.OO_PORT)
			BIB.OO_PIPE = current.Read('/OOo/pipe',BIB.OO_PIPE)
			BIB.OO_HILIGHT = current.ReadBool('/OOo/hilight',BIB.OO_HILIGHT)
			BIB.OO_AUTO_UPDATE = current.ReadBool('/OOo/auto_update',BIB.OO_AUTO_UPDATE)
			BIB.OO_CREATE_BIB = current.ReadBool('/OOo/create_bib',BIB.OO_CREATE_BIB)
		if current.HasGroup('printer'):	# printer settings
			if current.HasEntry('/printer/format'): BIB.PRINTER_FORMAT = eval(current.Read('/printer/format'))
			if current.HasEntry('/printer/colors'): BIB.PRINTER_COLORS = eval(current.Read('/printer/colors'))
			if current.HasEntry('/printer/style'): BIB.PRINTER_STYLE = eval(current.Read('/printer/style'))
			BIB.PRINTER_USE_OOo_FORMAT = current.ReadBool('/printer/use_ooo_format',BIB.PRINTER_USE_OOo_FORMAT)
		if current.HasGroup('shortcuts'):	# shortcuts settings
			if current.HasEntry('/shortcuts/shortcuts'): BIB.SHORTCUTS = eval(current.Read('/shortcuts/shortcuts'))
			if current.HasEntry('/shortcuts/shortfile'): BIB.SHORTFILE = eval(current.Read('/shortcuts/shortfile'))
		if current.HasGroup('path'):	# path settings
			BIB.FILES = current.Read('/path/files',BIB.FILES)

	def writeConfig(self,dbSave=True):
		"""Write config file. If dbSave=True save db config
		We use the current values in BIB module """
		# Now we save only when it is modified between BIB and BIBbase
		# and we delete the keys not needed any more
		# We put bDeleteGroupIfEmpty=False in DeleteEntry
		# because wx2.4 crash when the category becomes empty
		# it is ok under wx2.5 to put True
		current = self.myGet()
		#
		current.SetPath('/general')
		if BIBbase.WP != BIB.WP:
			current.Write('wp',BIB.WP)		# type of word processor OOo / mswDoc
		elif current.HasEntry('wp'):
			current.DeleteEntry('wp',False)	
		#
		current.SetPath('/display')
		if BIBbase.LIST_DISPLAY != BIB.LIST_DISPLAY:
			current.Write('fields',repr(BIB.LIST_DISPLAY))		# Fields displayed in reflist wxListCtrl
		else:
			if current.HasEntry('fields'): current.DeleteEntry('fields',False)
		#
		if BIBbase.LIST_ORDER != BIB.LIST_ORDER:
			current.Write('order',BIB.LIST_ORDER)				# Display order in reflist
		else:
			if current.HasEntry('order'): current.DeleteEntry('order',False)
		#
		if BIBbase.LIST_COL_SIZE != BIB.LIST_COL_SIZE:
			current.Write('col_size', repr(BIB.LIST_COL_SIZE) )	# tuple of column sizes
		else:
			if current.HasEntry('col_size'): current.DeleteEntry('col_size',False)
		#
		if BIBbase.DISPLAY_FORMAT != BIB.DISPLAY_FORMAT:
			current.WriteInt('display_format',BIB.DISPLAY_FORMAT)		# True if we use HTML format to display ref in the bottom-right panel
		else:
			if current.HasEntry('display_format'): current.DeleteEntry('display_format',False)
		#
		if BIBbase.KEY_COLOR != BIB.KEY_COLOR:
			current.Write('key_color',repr(BIB.KEY_COLOR.Get()))		# color display of keys (Title, AUthor, etc..) in textCtrl-1
		else:
			if current.HasEntry('key_color'): current.DeleteEntry('key_color',False)
		#
		if BIBbase.TEXT_COLOR != BIB.TEXT_COLOR:
			current.Write('text_color',repr(BIB.TEXT_COLOR.Get()))	# color display of text (Title, AUthor, etc..) in textCtrl-1
		else:
			if current.HasEntry('text_color'): current.DeleteEntry('text_color',False)
		#
		if BIBbase.EDIT != BIB.EDIT:
			current.Write('ref_editor',repr(BIB.EDIT))			# ReferenceEditor + Reference display in panel bottom-right of Main window
		else:
			if current.HasEntry('ref_editor'): current.DeleteEntry('ref_editor',False)
		#
		current.SetPath('/search')
		if BIBbase.BIB_SEARCH_FIELDS != BIB.BIB_SEARCH_FIELDS:
			current.Write('fields',repr(BIB.BIB_SEARCH_FIELDS))
		else:
			if current.HasEntry('fields'): current.DeleteEntry('fields',False)
		#
		current.SetPath('/db')
		if BIBbase.DB_STARTUP != BIB.DB_STARTUP:
			current.WriteInt('last_db',BIB.DB_STARTUP)
		else:
			if current.HasEntry('last_db'): current.DeleteEntry('last_db',False)
		#
		if BIBbase.STORE_PASSWD != BIB.STORE_PASSWD:
			current.WriteBool('store_passwd',BIB.STORE_PASSWD)
		else:
			if current.HasEntry('store_passwd'): current.DeleteEntry('store_passwd',False)
		#
		if BIBbase.DB_TYPE != BIB.DB_TYPE:
			current.Write('type',BIB.DB_TYPE)
		else:
			if current.HasEntry('type'): current.DeleteEntry('type',False)
		#
		if BIBbase.DUPLICATES_TEST != BIB.DUPLICATES_TEST:
			current.WriteBool('duplicates_test',BIB.DUPLICATES_TEST)
		else:
			if current.HasEntry('duplicates_test'): current.DeleteEntry('duplicates_test',False)
		#
		if BIBbase.DUPLICATES != BIB.DUPLICATES:
			current.Write('duplicates',repr(BIB.DUPLICATES))
		else:
			if current.HasEntry('duplicates'): current.DeleteEntry('duplicates',False)
		#
		if BIBbase.DUPLICATES_CASE != BIB.DUPLICATES_CASE:
			current.WriteBool('duplicates_case',BIB.DUPLICATES_CASE)
		else:
			if current.HasEntry('duplicates_case'): current.DeleteEntry('duplicates_case',False)
		#
		if BIBbase.DUPLICATES_KEEP_OLD != BIB.DUPLICATES_KEEP_OLD:
			current.WriteBool('duplicates_keep_old',BIB.DUPLICATES_KEEP_OLD)
		else:
			if current.HasEntry('duplicates_keep_old'): current.DeleteEntry('duplicates_keep_old',False)
		#
		if dbSave:					# we save db
			if BIB.DB_TYPE == 'MySQL':
				current.SetPath('/mysql')
				if BIBbase.DB_NAME != BIB.DB_NAME:
					current.Write('name',BIB.DB_NAME)
				else:
					if current.HasEntry('name'): current.DeleteEntry('name',False)
				#
				if BIBbase.USER != BIB.USER:
					current.Write('user',BIB.USER)
				else:
					if current.HasEntry('user'): current.DeleteEntry('user',False)
				#
				if BIBbase.PASSWORD != BIB.PASSWORD:
					current.Write('passwd',BIB.PASSWORD)	# PASSWORD='' if not to be stored
				else:
					if current.HasEntry('passwd'): current.DeleteEntry('passwd',False)
				#
				if BIBbase.HOST != BIB.HOST:
					current.Write('host',BIB.HOST)
				else:
					if current.HasEntry('host'): current.DeleteEntry('host',False)
				#
				if BIBbase.PORT != BIB.PORT:
					current.WriteInt('port',BIB.PORT)
				else:
					if current.HasEntry('port'): current.DeleteEntry('port',False)
				#
				if BIBbase.SOCKET != BIB.SOCKET:
					current.Write('socket',BIB.SOCKET)
				else:
					if current.HasEntry('socket'): current.DeleteEntry('socket',False)
				#
			elif BIB.DB_TYPE == 'SQLite':
				current.SetPath('/sqlite')
				if BIBbase.SQLiteFile != BIB.SQLiteFile:
					current.Write('file',BIB.SQLiteFile)
				else:
					if current.HasEntry('file'): current.DeleteEntry('file',False)
				#
				if BIBbase.SQLiteUSER != BIB.SQLiteUSER:
					current.Write('user',BIB.SQLiteUSER)
				else:
					if current.HasEntry('user'): current.DeleteEntry('user',False)
		#
		current.SetPath('/window')
		if BIBbase.WX != BIB.WX:
			current.WriteInt('x',BIB.WX)
		else:
			if current.HasEntry('x'): current.DeleteEntry('x',False)
		#
		if BIBbase.WY != BIB.WY:
			current.WriteInt('y',BIB.WY)
		else:
			if current.HasEntry('y'): current.DeleteEntry('y',False)
		#
		if BIBbase.WIDTH != BIB.WIDTH:
			current.WriteInt('width',BIB.WIDTH)
		else:
			if current.HasEntry('width'): current.DeleteEntry('width',False)
		#
		if BIBbase.HEIGHT != BIB.HEIGHT:
			current.WriteInt('height',BIB.HEIGHT)
		else:
			if current.HasEntry('height'): current.DeleteEntry('height',False)
		#
		if BIBbase.SASH_LIST != BIB.SASH_LIST:
			current.WriteInt('sash_list',BIB.SASH_LIST)
		else:
			if current.HasEntry('sash_list'): current.DeleteEntry('sash_list',False)
		#
		if BIBbase.SASH_KEYTREE != BIB.SASH_KEYTREE:
			current.WriteInt('sash_keytree',BIB.SASH_KEYTREE)
		else:
			if current.HasEntry('sash_keytree'): current.DeleteEntry('sash_keytree',False)
		#
		if BIBbase.REF_Ed_WIDTH != BIB.REF_Ed_WIDTH:
			current.WriteInt('ref_width',BIB.REF_Ed_WIDTH)
		else:
			if current.HasEntry('ref_width'): current.DeleteEntry('ref_width',False)
		#
		if BIBbase.REF_Ed_X != BIB.REF_Ed_X:
			current.WriteInt('ref_x',BIB.REF_Ed_X)
		else:
			if current.HasEntry('ref_x'): current.DeleteEntry('ref_x',False)
		#
		if BIBbase.REF_Ed_Y != BIB.REF_Ed_Y:
			current.WriteInt('ref_y',BIB.REF_Ed_Y)
		else:
			if current.HasEntry('ref_y'): current.DeleteEntry('ref_y',False)
		#
		if BIBbase.IMPORT_WIDTH != BIB.IMPORT_WIDTH:
			current.WriteInt('import_width',BIB.IMPORT_WIDTH)
		else:
			if current.HasEntry('import_width'): current.DeleteEntry('import_width',False)
		#
		if BIBbase.IMPORT_HEIGHT != BIB.IMPORT_HEIGHT:
			current.WriteInt('import_height',BIB.IMPORT_HEIGHT)
		else:
			if current.HasEntry('import_height'): current.DeleteEntry('import_height',False)
		#
		if BIBbase.IMPORT_X != BIB.IMPORT_X:
			current.WriteInt('import_x',BIB.IMPORT_X)
		else:
			if current.HasEntry('import_x'): current.DeleteEntry('import_x',False)
		#
		if BIBbase.IMPORT_Y != BIB.IMPORT_Y:
			current.WriteInt('import_y',BIB.IMPORT_Y)
		else:
			if current.HasEntry('import_y'): current.DeleteEntry('import_y',False)
		#
		if BIBbase.IMPORT_FORMAT != BIB.IMPORT_FORMAT:
			current.WriteInt('import_format',BIB.IMPORT_FORMAT)
		else:
			if current.HasEntry('import_format'): current.DeleteEntry('import_format',False)
		#
		if BIBbase.SEARCH_WIDTH != BIB.SEARCH_WIDTH:
			current.WriteInt('search_width',BIB.IMPORT_WIDTH)
		else:
			if current.HasEntry('search_width'): current.DeleteEntry('search_width',False)
		#
		if BIBbase.SEARCH_HEIGHT != BIB.SEARCH_HEIGHT:
			current.WriteInt('search_height',BIB.SEARCH_HEIGHT)
		else:
			if current.HasEntry('search_height'): current.DeleteEntry('search_height',False)
		#
		if BIBbase.SEARCH_X != BIB.SEARCH_X:
			current.WriteInt('search_x',BIB.SEARCH_X)
		else:
			if current.HasEntry('search_x'): current.DeleteEntry('search_x',False)
		#
		if BIBbase.SEARCH_Y != BIB.SEARCH_Y:
			current.WriteInt('search_y',BIB.SEARCH_Y)
		else:
			if current.HasEntry('search_y'): current.DeleteEntry('search_y',False)
		#
		if BIBbase.SEARCH_WIDTH != BIB.PUBMED_WIDTH:
			current.WriteInt('pubmed_width',BIB.IMPORT_WIDTH)
		else:
			if current.HasEntry('pubmed_width'): current.DeleteEntry('pubmed_width',False)
		#
		if BIBbase.PUBMED_HEIGHT != BIB.PUBMED_HEIGHT:
			current.WriteInt('pubmed_height',BIB.PUBMED_HEIGHT)
		else:
			if current.HasEntry('pubmed_height'): current.DeleteEntry('pubmed_height',False)
		#
		if BIBbase.PUBMED_X != BIB.PUBMED_X:
			current.WriteInt('pubmed_x',BIB.PUBMED_X)
		else:
			if current.HasEntry('pubmed_x'): current.DeleteEntry('pubmed_x',False)
		#
		if BIBbase.PUBMED_Y != BIB.PUBMED_Y:
			current.WriteInt('pubmed_y',BIB.PUBMED_Y)
		else:
			if current.HasEntry('pubmed_y'): current.DeleteEntry('pubmed_y',False)
		#
		if BIBbase.PUBMED_NB != BIB.PUBMED_NB:
			current.WriteInt('pubmed_nb',BIB.PUBMED_NB)
		else:
			if current.HasEntry('pubmed_nb'): current.DeleteEntry('pubmed_nb',False)
		#
		current.SetPath('/OOo')		# OOo settings
		if BIBbase.OO_CON_TYPE != BIB.OO_CON_TYPE:
			current.WriteInt('con_type',BIB.OO_CON_TYPE)
		else:
			if current.HasEntry('con_type'): current.DeleteEntry('con_type',False)
		#
		if BIBbase.OO_HOST != BIB.OO_HOST:
			current.Write('host',BIB.OO_HOST)
		else:
			if current.HasEntry('host'): current.DeleteEntry('host',False)
		#
		if BIBbase.OO_PORT != BIB.OO_PORT:
			current.WriteInt('port',BIB.OO_PORT)
		else:
			if current.HasEntry('port'): current.DeleteEntry('port',False)
		#
		if BIBbase.OO_PIPE != BIB.OO_PIPE:
			current.Write('pipe',BIB.OO_PIPE)
		else:
			if current.HasEntry('pipe'): current.DeleteEntry('pipe',False)
		#
		if BIBbase.OO_HILIGHT != BIB.OO_HILIGHT:
			current.WriteBool('hilight',BIB.OO_HILIGHT)
		else:
			if current.HasEntry('hilight'): current.DeleteEntry('hilight',False)
		#
		if BIBbase.OO_AUTO_UPDATE != BIB.OO_AUTO_UPDATE:
			current.WriteBool('auto_update',BIB.OO_AUTO_UPDATE)
		else:
			if current.HasEntry('auto_update'): current.DeleteEntry('auto_update',False)
		#
		if BIBbase.OO_CREATE_BIB != BIB.OO_CREATE_BIB:
			current.WriteBool('create_bib',BIB.OO_CREATE_BIB)
		else:
			if current.HasEntry('create_bib'): current.DeleteEntry('create_bib',False)
		#
		current.SetPath('/printer')		# printer settings
		if BIBbase.PRINTER_FORMAT != BIB.PRINTER_FORMAT:
			current.Write('format',repr(BIB.PRINTER_FORMAT))
		else:
			if current.HasEntry('format'): current.DeleteEntry('format',False)
		#
		if BIBbase.PRINTER_COLORS != BIB.PRINTER_COLORS:
			current.Write('colors',repr(BIB.PRINTER_COLORS))
		else:
			if current.HasEntry('colors'): current.DeleteEntry('colors',False)
		#
		if BIBbase.PRINTER_STYLE != BIB.PRINTER_STYLE:
			current.Write('style',repr(BIB.PRINTER_STYLE))
		else:
			if current.HasEntry('style'): current.DeleteEntry('style',False)
		#
		if BIBbase.PRINTER_USE_OOo_FORMAT != BIB.PRINTER_USE_OOo_FORMAT:
			current.WriteBool('use_ooo_format',BIB.PRINTER_USE_OOo_FORMAT)
		else:
			if current.HasEntry('use_ooo_format'): current.DeleteEntry('use_ooo_format',False)
		#
		current.SetPath('/shortcuts')	# shortcuts settings
		if BIBbase.SHORTCUTS != BIB.SHORTCUTS:
			current.Write('shortcuts',repr(BIB.SHORTCUTS))
		else:
			if current.HasEntry('shortcuts'): current.DeleteEntry('shortcuts',False)
		#
		if BIBbase.SHORTFILE != BIB.SHORTFILE:
			current.Write('shortfile',repr(BIB.SHORTFILE))
		else:
			if current.HasEntry('fields'): current.DeleteEntry('shortfile',False)
		#
		current.SetPath('/path')	# path settings
		if BIBbase.FILES != BIB.FILES:
			current.Write('files',BIB.FILES)
		else:
			if current.HasEntry('files'): current.DeleteEntry('files',False)
		#
		current.Flush()	# save the config changes

#---------------------- Shortcuts ----------------------------------------------
	def readShortcut(self,n):
		"""Load shortcut number n"""
		try:
			f = file(BIB.SHORTFILE[n],'r')
			short = cPickle.load(f)		# we unpickle the file then check that it is a list of strings
			f.close()
			if type(short) != list: raise "notalist"
			else:
				for i in short:
					if type(i) not in (unicode,str): raise "notastring"
		except:
			short = []
		BIB.SHORT[BIB.SHORTCUTS[n]] = short

	def writeShortcuts(self):
		for index in xrange(len(BIB.SHORTCUTS)):
			if BIB.SHORT.has_key(BIB.SHORTCUTS[index]):
				f = file(BIB.SHORTFILE[index],'w')
				cPickle.dump( BIB.SHORT[BIB.SHORTCUTS[index]] , f)
				f.close()

#---------------------- OOo styles ---------------------------------------------
	def writeStyle(self,filename):
		"""Write the filename of the user Style
		key = basename, value = path+basename"""
		current = self.myGet()
		current.SetPath('/styles')
		current.Write(os.path.basename(filename),filename)

	def deleteStyle(self,filename):
		current = self.myGet()
		current.SetPath('/styles')
		current.DeleteEntry(os.path.basename(filename),False)

	def readStyles(self):
		"""Return a tuple of the filenames of user styles."""
		current = self.myGet()
		if current.HasGroup('/styles'):
			current.SetPath('/styles')
			styles=[]
			flag,s,cookie = current.GetFirstEntry()
			while flag:
				styles.append(current.Read(s))
				flag,s,cookie = current.GetNextEntry(cookie)
			return tuple(styles)
		else:
			return ()

	def writeCurrentStyle(self,style):
		"""Save the Current style filename under key style/current"""
		current = self.myGet()
		current.Write('/style/current',style)

	def getCurrentStyle(self):
		"""Return the Current style"""
		current = self.myGet()
		return current.Read('/style/current')

# --------------------------- FirstStart ---------------------------------------
	def firstStartDone(self):
		"""Set to False after first start"""
		current = self.myGet()
		current.WriteBool('/general/firststart',False)
		current.Write('/general/config_version',BIB.BIBUS_VERSION)
		current.Flush()

	def userDefined(self,dbType):
		"""Return true if the username was already defined in the config"""
		current = self.myGet()
		return current.HasEntry('/%s/user'%dbType)
