# Copyright 2004,2005 Pierre Martineau <pmartino@users.sourceforge.net>
# This file is part of Bibus, a bibliographic database that can
# work together with OpenOffice.org to generate bibliographic indexes.
#
# Bibus is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bibus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bibus; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Part of the Wizard code has been copied from the example "Simple Wizard" included with
# wx.Python
#
import getpass, os, sys, ConfigParser, shutil
#
import wx, wx.wizard
import BIB
from FirstTimeWizard_WP import FirstTimeWizard_WP

# ----------------------------------------------------------------
class TitledPage(wx.wizard.WizardPageSimple):
	def __init__(self, parent, title):
		wx.wizard.WizardPageSimple.__init__(self, parent)
		self.sizer = self.__makePageTitle(title)

	def __makePageTitle(self, title):
		sizer = wx.BoxSizer(wx.VERTICAL)
		self.SetSizer(sizer)
		title = wx.StaticText(self, -1, title)
		title.SetFont(wx.Font(18, wx.SWISS, wx.NORMAL, wx.BOLD))
		sizer.Add(title, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
		sizer.Add(wx.StaticLine(self, -1), 0, wx.EXPAND|wx.ALL, 5)
		return sizer
# ----------------------------------------------------------------
class FirstStart(wx.wizard.Wizard):
	"""This is the Frame that open on the firstconnection"""
	def __init__(self,parent):
	# Create the wizard and the pages
		self.sqliteversion='2'		# version of sqlite used (depend on the pysqlite module)
		self.ID_wiz = wx.NewId()
		self.parent = parent
		if not BIB.CONFIG.userDefined('sqlite'):
			try:
				BIB.USER = getpass.getuser()	# We try to get a default value if not already defined.
			except:
				BIB.USER = ''
		cp = ConfigParser.ConfigParser()
		#
		wx.wizard.Wizard.__init__(self,parent, self.ID_wiz, _("First Connection Wizard"))
		self.Show(1)
		pageWP = TitledPage(self, _("Welcome to Bibus !"))
		pageOOo = TitledPage(self, _("OpenOffice.org connection"))
		page0 = TitledPage(self, _("Database engine"))
		page1 = TitledPage(self, _("Setting username"))
		page2 = TitledPage(self, _("Creating or choosing the database"))
		self.pageWP = pageWP
		self.pageOOo = pageOOo
		self.page0 = page0
		self.page1 = page1
		self.page2 = page2
		# pageWP Layout. Choice of MS Word or OOo connection
		self.page_wp = FirstTimeWizard_WP(pageWP,-1,wizard = self)
		pageWP.sizer.Add( self.page_wp,1,wx.EXPAND )
		# pageOOo Layout
		tmp = wx.StaticText(pageOOo,-1,_("OpenOffice.org connection"))
		tmp.SetFont(wx.Font(14, wx.SWISS, wx.NORMAL, wx.BOLD))
		pageOOo.sizer.Add(tmp)
		pageOOo.sizer.Add(wx.StaticText(pageOOo,-1,\
_("\nBibus can directly insert citations in OpenOffice.org\n\
if you start OpenOffice.org in 'listening mode'.\n\n\n\
Do you want to activate this mode by default?\n\n\n\
A document will open in OpenOffice.org,\n\
Click on 'Accept UNO connections',\n\
Then restart OpenOffice.org.\n\
Under Windows, you must also quit the OpenOffice.org Quickstarter.\n\n\
")))
		#
		self.activateOOo = wx.Button(pageOOo,-1,_("Activate"))
		pageOOo.sizer.Add(self.activateOOo,0,wx.ALIGN_CENTER)
		# page0 Layout
		page0.sizer.Add(tmp)
		page0.sizer.Add(wx.StaticText(page0,-1,_("Bibus may use two database engines:")))
		page0.sizer.Add(wx.StaticText(page0,-1,_("MySQL or SQLite")),0,wx.ALIGN_CENTER)
		page0.sizer.Add((0,50))
		tmp = wx.StaticText(page0,-1,_("If you want to use MySQL"))
		tmp.SetFont(wx.Font(14, wx.SWISS, wx.NORMAL, wx.BOLD))
		page0.sizer.Add(tmp)
		page0.sizer.Add(wx.StaticText(page0,-1,_("- Click on Cancel")))
		page0.sizer.Add(wx.StaticText(page0,-1,_("- Follow the instructions in mysql_config.txt")))
		page0.sizer.Add((0,50))
		tmp = wx.StaticText(page0,-1,_("If you want to use SQLite"))
		tmp.SetFont(wx.Font(14, wx.SWISS, wx.NORMAL, wx.BOLD))
		page0.sizer.Add(tmp)
		page0.sizer.Add(wx.StaticText(page0,-1,_("- Click on Next")))
		# page1 Layout
		tmp = wx.StaticText(page1,-1,\
_("Bibus needs a USERNAME to identify yourself.\
\nPlease enter a name in the box below.\
\nYour login name might be a good idea"))
		#tmp.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.NORMAL))
		page1.sizer.Add(tmp)
		page1.sizer.Add((0,50))
		self.user=wx.TextCtrl(page1,-1,BIB.SQLiteUSER)
		page1.sizer.Add(self.user, 0, wx.EXPAND)
		# page2 Layout
		tmp = wx.StaticText(page2,-1,\
_("Bibus uses a file to store the bibliographic database.\
\nPlease enter in the box below the path to \
\nthe bibliographic database you want to use.\
\n- If the file EXIST, Bibus will \
\n	- not modify it\
\n	- assume it to be a correct Bibus database.\
\n- If the file DOES NOT EXIST, Bibus will \
\n	- create it\
\n	- use it as your bibliographic database."))
		page2.sizer1 = wx.BoxSizer(wx.HORIZONTAL)
		self.filen=wx.TextCtrl(page2,-1,BIB.SQLiteFile)
		self.db_file = wx.Button(page2,-1,_("Database file ..."))
		page2.sizer1.Add(self.db_file,0,wx.ALIGN_RIGHT)
		page2.sizer1.Add(self.filen, 1, wx.EXPAND)
		page2.sizer.Add(tmp)
		page2.sizer.Add((0,50))
		page2.sizer.Add(page2.sizer1, 0, wx.EXPAND)
		# Chaining the pages
		wx.wizard.WizardPageSimple_Chain(pageWP, pageOOo)
		wx.wizard.WizardPageSimple_Chain(pageOOo, page0)
		wx.wizard.WizardPageSimple_Chain(page0, page1)
		wx.wizard.WizardPageSimple_Chain(page1, page2)
		self.FitToPage(page0)
		self.Layout()
		#
		wx.EVT_BUTTON(self, self.activateOOo.GetId(), self.set_connection)
		wx.wizard.EVT_WIZARD_PAGE_CHANGED(self, self.ID_wiz, self.OnWizPageChanged)
		wx.wizard.EVT_WIZARD_CANCEL(self, self.ID_wiz, self.OnWizCancel)
		wx.EVT_BUTTON(self, self.db_file.GetId(), self.OnFileChoice)
		wx.wizard.EVT_WIZARD_FINISHED(self, self.ID_wiz, self.OnWizFinish)

	def Run(self):
		try:
			import uno
			pyunoOK = True
		except ImportError:
			pyunoOK = False
		#
		try:
			import win32com
			wordOK = True
		except ImportError:
			wordOK = False
		#
		if pyunoOK or wordOK:	# word or OOo connection is possible
			if not wordOK: self.page_wp.justOOo()
			if not pyunoOK: self.page_wp.justWord()
			self.FitToPage(self.pageWP)
			self.RunWizard(self.pageWP)
		else:					# no connection
			self.RunWizard(self.page0)

	def OnWizPageChanged(self, evt):
		page = evt.GetPage()
		if page == self.page1:
			try:
				try:
					from pysqlite2 import dbapi2 as sqlite
				except ImportError:
					import sqlite
				if sqlite.version_info < (1,1,0):
					self.sqliteversion = '1_0'
				elif sqlite.version_info < (2,0,0):
					self.sqliteversion = '1_1'
				else:
					self.sqliteversion = '2'
			except ImportError:
				wx.MessageBox(_("Sorry but the SQLite python extension is not available.\nRead installation.txt"),_('Error'),wx.ICON_ERROR|wx.OK)
				self.Close()
				
	def setOOoActivate(self):
		"""We don't go to the activate page if selection is not OOo"""
		if BIB.WP != 'OOo':
			wx.wizard.WizardPageSimple_Chain(self.pageWP, self.page0)
		else:
			wx.wizard.WizardPageSimple_Chain(self.pageWP, self.pageOOo)

	def OnWizFinish(self, evt):
		# we are leaving the wizard by clicking the Finish button
		# we first check that the connection is working then we return the dbBib connection
		# before leaving
		BIB.DB_TYPE = 'SQLite'
		BIB.SQLiteFile = self.filen.GetValue()
		BIB.SQLiteUSER = self.user.GetValue()
		self.createSQLiteDB()
		self.NoFirstStart()
		BIB.CONFIG.writeConfig(not bool(BIB.DB_STARTUP))

	def createSQLiteDB(self):
		"""create a sqlite database"""
		if not os.path.exists(BIB.SQLiteFile):
			shutil.copyfile( os.path.join(BIB.SOURCEDIR,'db_models','biblio.pysqlite%s'%self.sqliteversion), BIB.SQLiteFile )
		elif self.sqliteversion in ('1_1','2'):	# new database format and we use a previous file => warn the user
			fileDB = open(BIB.SQLiteFile,'r')
			if fileDB.read(15) != 'SQLite format 3':
				wx.MessageBox(_("It seems that your database is not compatible with your PySQLite version.\nIf it does not work,\nsee the SQLite site about database format change in SQLite3 http://www.sqlite.org/version3.html"),_('Error'),wx.ICON_ERROR|wx.OK)
			fileDB.close()
		elif self.sqliteversion == '1_0':
			fileDB = open(BIB.SQLiteFile,'r')
			if fileDB.read(47) != '** This file contains an SQLite 2.1 database **':
				wx.MessageBox(_("It seems that your database is not compatible with your PySQLite version.\nIf it does not work,\nsee the SQLite site about database format change in SQLite3 http://www.sqlite.org/version3.html"),_('Error'),wx.ICON_ERROR|wx.OK)
			fileDB.close()
		else:
			print "What are we doing here ? line 192 in FirstStart.py"	

	def OnWizCancel(self, evt):
		BIB.DB_TYPE = 'MySQL'
		self.NoFirstStart()

	def NoFirstStart(self):
		BIB.FIRST_START = False
		BIB.CONFIG.firstStartDone()

	def OnFileChoice(self,event):
		if BIB.SQLiteFile:
			default = BIB.SQLiteFile
		else:
			default = 'biblioDB.sqlite'
		filename = wx.FileSelector(_('SQlite database file'),default_filename = default, flags = wx.SAVE )
		self.filen.SetValue(filename)

	def set_connection(self,evt):
		"""We start OOo with the UnoConnectionListener.sxd"""
		# command to start soffice
		if os.name == 'posix':
			os.spawnl(os.P_NOWAIT,os.path.join(BIB.OOPATH,'soffice'),"soffice",os.path.join(BIB.SOURCEDIR,'Setup','UnoConnectionListener.sxd'))
		else:
			os.spawnl(os.P_NOWAIT,os.path.join(BIB.OOPATH,'soffice.exe'),"soffice.exe",'"'+ os.path.join(BIB.SOURCEDIR,'Setup','UnoConnectionListener.sxd') +'"')
