# Copyright 2005 Nigel Sim <nigel.sim@jcu.edu.au>
# This file is part of Bibus, a bibliographic database that can
# work together with OpenOffice.org to generate bibliographic indexes.
#
# Bibus is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bibus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bibus; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# BibTeX format
# from http://en.wikipedia.org/wiki/BibTeX
#
from __future__ import generators		# to be removed in python 2.3
import BIB

class importRef(object):
	"""Class is iterable. Return records one by one with None for the id (first field)."""
 	Type={
	'article':'ARTICLE',
	'book':'BOOK',
	'booklet':'BOOKLET',
	'conference':'CONFERENCE',
	'inbook':'INBOOK',
	'incollection':'INCOLLECTION',
	'inproceedings':'INPROCEEDINGS',
	'manual':'MANUAL',
	'mastersthesis':'MASTERTHESIS',
	'misc':'MISC',
	'phdthesis':'PHDTHESIS',
	'proceedings':'PROCEEDINGS',
	'techreport':'TECHREPORT',
	'unpublished':'UNPUBLISHED'}

	"""return a list of the record fields using
	('Identifier', 'Bibliographic_Type', 'Address', 'Annote', 'Author', 'Booktitle', 'Chapter', 'Edition', 		'Editor','HowPublished', 'Institution', 'Journal', 'Month', 'Note', 'Number', 'Organizations', 'Pages', 'Publisher',
	'School','Series', 'Title', 'Report_Type', 'Volume', 'Year', 'URL', 'Custom1', 'Custom2', 'Custom3', 'Custom4', 'Custom5', 		'ISBN','Abstract')"""
	Fields={
	'address':3,
	'annote':4,
	'author':5,
	'booktitle':6,
	'chapter':7,
	'edition':8,
	'editor':9,
	'howpublished':10,
	'institution':11,
	'journal':12,
	'month':13,
	'note':14,
	'number':15,
	'organization':16,
	'pages':17,
	'publisher':18,
	'school':19,
	'series':20,
	'title':21,
	'volume':23,
	'year':24,
	'url':25,
	'doi':26,
	'location':27,
	'issn':31}

	def __init__(self,infile):
		self.infile = infile	# must be a file type. Need a readline() function.

	def __iter__(self):
		"""Generator of records. for record in <instance>: ... """
		record  = []
		for i in range(33):
			record.append(None)

		name = None
		index = None

		line = self.infile.readline()
		while line != '':
			for l in line:
				if l == '@' and record[1] == None:
					index = 1
					record[index] = ""
				elif l =='\r' or l == '\n':
					continue # Do nothing
				elif l == ',' and index == None:
					name = "" # Reset the name if we find a comma, which is a field seperator
				elif l == '{' and record[1].__class__ != int:
					print record[1]
					print self.Type[record[1]]
					record[1] = BIB.BIBLIOGRAPHIC_TYPE[self.Type[record[1]]] # Change the BibTeX type string to a Bibus type
					index = None
				elif (l == '"' or l == '{' ) and index != None and record[index] == None:
					record[index] = ""
				elif (l == '"' or l == '}') and index != None and record[index] != None:
					index = None
				elif l == '=' and index == None:
					index = self.Fields[name.strip()]
					#print "New index is "+`index`
				elif index != None and record[index] != None:
					record[index] = record[index] + l
					#print record[index]
				elif index == None:
					if name == None:
						name = ""
					name = name + l
				else:
					print "uh oh"
			line = self.infile.readline()
		else:
			# Get rid of any remaining None objects
			for i in range(33):
				if i > 0 and record[i] == None:
					record[i] = ""

			yield record
