# Copyright 2004,2005 Pierre Martineau <pmartino@users.sourceforge.net>
# This file is part of Bibus, a bibliographic database that can
# work together with OpenOffice.org to generate bibliographic indexes.
#
# Bibus is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bibus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bibus; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
from wxPython.wx import *
import getpass, sys
import BIB
from Pref_Printing import Pref_Printing
from Pref_Editor import Pref_Editor
from Pref_Shortcuts import Pref_Shortcuts
from Pref_Paths import Pref_Paths
from Pref_Duplicates import Pref_Duplicates
from Pref_DB import Pref_DB
from Pref_Connection import Pref_Connection

class BibPref(wxDialog):
	def __init__(self, db, *args, **kwds):
		#kwds["style"] = wxDEFAULT_FRAME_STYLE
		wxDialog.__init__(self, *args, **kwds)
		BIB.CONFIG.readConfig()	# we read the current config
		#
		self.colors = {_('Black'):wxBLACK,_('White'):wxWHITE,_('Red'):wxRED,_('Blue'):wxBLUE,_('Green'):wxGREEN,_('Cyan'):wxCYAN,_('Light_grey'):wxLIGHT_GREY}
		#
		s1 = wxBoxSizer(wxVERTICAL)	# top = notebook ; bottom = buttons
		s2 = wxBoxSizer(wxHORIZONTAL)	# to put buttons in it
		# buttons
		self.button_ok = wxButton(self, wxID_OK, _("OK"))
		self.button_cancel =wxButton(self, wxID_CANCEL, _("Cancel"))
		s2.Add(self.button_cancel,0,wxALL,5)
		s2.Add(self.button_ok,0,wxALL,5)
		self.button_ok.SetDefault()
		# notebook
		self.nb = wxNotebook(self,-1,style=wxNB_TOP)
		self.prefDisplay = wxPanel(self.nb,-1)
		self.prefPrinting = Pref_Printing(self.nb,-1)
		self.prefEditor = Pref_Editor(self.nb,-1)
		self.prefSearch = wxPanel(self.nb,-1)
		self.prefDuplicates = Pref_Duplicates(self.nb,-1, (BIB.DUPLICATES_TEST,BIB.DUPLICATES,BIB.DUPLICATES_CASE,BIB.DUPLICATES_KEEP_OLD) )
		self.prefShortcuts = Pref_Shortcuts(db,self.nb,-1)
		self.prefPaths = Pref_Paths(self.nb,-1)
		self.prefDB = Pref_DB(self.nb,-1, (BIB.DB_STARTUP,BIB.DB_TYPE,BIB.STORE_PASSWD,BIB.DB_NAME,BIB.USER,BIB.PASSWORD,BIB.HOST,BIB.PORT,BIB.SOCKET,BIB.SQLiteFile,BIB.SQLiteUSER) )
		self.prefConnection = Pref_Connection(self.nb,-1, (BIB.WP,BIB.OO_CON_TYPE,BIB.OO_PIPE,BIB.OO_HOST,BIB.OO_PORT))
		self.nb.AddPage(self.prefDisplay,_("Display"))
		self.nb.AddPage(self.prefPrinting,_("Printing"))
		self.nb.AddPage(self.prefSearch,_("Search"))
		self.nb.AddPage(self.prefDuplicates,_("Duplicates"))
		self.nb.AddPage(self.prefEditor,_("Editor"))
		self.nb.AddPage(self.prefShortcuts,_("Shortcuts"))
		self.nb.AddPage(self.prefPaths,_("Paths"))
		self.nb.AddPage(self.prefDB,_("Database"))
		self.nb.AddPage(self.prefConnection,_("Word Processor"))
		# page setup
		self.__setup_display()
		self.__setup_search()
		# layout
		if wxVERSION[:2] >= (2,6):	# wxNotebookSizer no more needed with wxPython2.6
			s1.Add(self.nb, 1, wxEXPAND|wxALL,5)
		else:
			s1.Add(wxNotebookSizer(self.nb), 1, wxEXPAND|wxALL,5)
		s1.Add(s2,0,wxALIGN_CENTER_HORIZONTAL,5)
		s1.Fit(self)
		self.SetSizer(s1)
		self.Layout()
		self.CenterOnParent()
		#
		self.__set_event()

	def __setup_display(self):
		s1 = wxBoxSizer(wxVERTICAL)	# top = fields choice ; medium = sort order ; bottom = color ref display
		s2 = wxStaticBoxSizer(wxStaticBox(self.prefDisplay,-1,_("Reference list")),wxHORIZONTAL)	# top
		s2_available = wxBoxSizer(wxVERTICAL)	# listBox left
		s2_selected = wxBoxSizer(wxVERTICAL)	# listBox right
		s2_sort = wxBoxSizer(wxHORIZONTAL)	# to pack choice sort + title
		s3 = wxBoxSizer(wxVERTICAL)	# buttons add and remove up and down
		s4 = wxStaticBoxSizer(wxStaticBox(self.prefDisplay,-1,_('Reference display')),wxVERTICAL)	# buttons colors
		s5 = wxFlexGridSizer(3,2,3,3)	# layout of colors choices
		s5.AddGrowableCol(0)
		#
		tmpstr = list(BIB.BIB_FIELDS)
		for item in BIB.LIST_DISPLAY:
			del(tmpstr[tmpstr.index(item)])	# remove already selected items
		self.fields = wxListBox(self.prefDisplay,-1,choices=tmpstr,style=wxLB_SINGLE)
		self.fieldsChoice = wxListBox(self.prefDisplay,-1,choices=list(BIB.LIST_DISPLAY),style=wxLB_SINGLE)
		self.b_add = wxButton(self.prefDisplay,-1, _('Add'))
		self.b_remove = wxButton(self.prefDisplay,-1, _('Remove'))
		self.b_up = wxButton(self.prefDisplay,-1, _('Up'))
		self.b_down = wxButton(self.prefDisplay,-1, _('Down'))
		#
		self.sort_order = wxChoice(self.prefDisplay,-1,choices=list(BIB.BIB_FIELDS))
		self.sort_order.SetStringSelection(BIB.LIST_ORDER)		# selected previous sort order or the first one if not present
		#
		self.refformatpanel = wxPanel(self.prefDisplay,-1)
		self.key_color = wxChoice(self.refformatpanel,-1,choices=self.colors.keys())
		self.text_color = wxChoice(self.refformatpanel,-1,choices=self.colors.keys())
		for i in self.colors.iteritems():
			if BIB.KEY_COLOR == i[1]:
				self.key_color.SetStringSelection(i[0])
				break
		for i in self.colors.iteritems():
			if BIB.TEXT_COLOR == i[1]:
				self.text_color.SetStringSelection(i[0])
				break
		#
		s5.Add(wxStaticText(self.refformatpanel,-1,_('Field name color')),0,wxALIGN_CENTER_VERTICAL)
		s5.Add(self.key_color,0,0)
		s5.Add(wxStaticText(self.refformatpanel,-1,_('Field color')),0,wxALIGN_CENTER_VERTICAL)
		s5.Add(self.text_color,0,0)
		self.refformatpanel.SetSizer(s5)
		s4.Add(self.refformatpanel)
		#
		s3.Add(self.b_add,0,0)
		s3.Add(self.b_remove,0,0)
		s3.Add((0,5),0,0)
		s3.Add(self.b_up,0,0,0)
		s3.Add(self.b_down,0,0)
		#
		s2_available.Add(wxStaticText(self.prefDisplay,-1,_('Reference fields')),0,wxALL,5)
		s2_available.Add(self.fields,1,wxEXPAND)
		s2.Add(s2_available,1,wxEXPAND)
		s2.Add(s3,0,wxALL|wxALIGN_CENTER_VERTICAL,5)
		s2_selected.Add(wxStaticText(self.prefDisplay,-1,_('Displayed fields')),0,wxALL,5)
		s2_selected.Add(self.fieldsChoice,1,wxEXPAND)
		s2_sort.Add(wxStaticText(self.prefDisplay,-1,_("Sort order")),0,wxALIGN_CENTER_VERTICAL)
		s2_sort.Add(self.sort_order,0,wxALIGN_RIGHT|wxALL,5)
		s2_selected.Add(s2_sort,0,wxEXPAND)
		s2.Add(s2_selected,1,wxEXPAND)
		#
		s1.Add(s2,1,wxEXPAND)
		s1.Add(s4,0,wxEXPAND)
		#
		self.prefDisplay.SetAutoLayout(True)
		self.prefDisplay.SetSizer(s1)

	def __setup_search(self):
		s1 = wxBoxSizer(wxVERTICAL)
		s1.Add(wxStaticText(self.prefSearch,-1,_('Main fields searched')),0,wxALL,5)
		self.searchFields = wxListBox(self.prefSearch,-1,choices=list(BIB.BIB_FIELDS),style=wxLB_MULTIPLE)
		s1.Add(self.searchFields,1,wxEXPAND|wxALL,5)
		for tmpstr in BIB.BIB_SEARCH_FIELDS:
			self.searchFields.SetStringSelection(tmpstr,True)
		self.prefSearch.SetAutoLayout(True)
		self.prefSearch.SetSizer(s1)

	def __set_event(self):
		EVT_BUTTON(self,wxID_OK,self.onOK)
		EVT_BUTTON(self,wxID_CANCEL,self.onCloseWindow)
		EVT_CLOSE(self, self.onCloseWindow)
		EVT_BUTTON(self,self.b_add.GetId(),self.add)
		EVT_BUTTON(self,self.b_remove.GetId(),self.remove)
		EVT_BUTTON(self,self.b_up.GetId(),self.up)
		EVT_BUTTON(self,self.b_down.GetId(),self.down)

	def onOK(self,event):
		# fields to display
		selected_fields=[]
		for i in range(self.fieldsChoice.GetCount()):
			selected_fields.append(self.fieldsChoice.GetString(i))
		BIB.LIST_DISPLAY = tuple(selected_fields)
		# we resize list_display
		if len(BIB.LIST_COL_SIZE) >= len(BIB.LIST_DISPLAY):
			BIB.LIST_COL_SIZE = BIB.LIST_COL_SIZE[:len(BIB.LIST_DISPLAY)]
		else:
			BIB.LIST_COL_SIZE = BIB.LIST_COL_SIZE + (BIB.LIST_COL_SIZE_DEFAULT,)*(len(BIB.LIST_DISPLAY) - len(BIB.LIST_COL_SIZE))
		self.GetParent().reflist.resizeList()
		if self.GetParent().keytree.GetSelection():
			self.GetParent().KeySelect(self.GetParent().keytree.GetSelection())
		# sort order
		BIB.LIST_ORDER = self.sort_order.GetStringSelection()
		try:
			self.GetParent().resizeList()	# resize column list in reflist
			if self.GetParent().keytree.GetSelection():
				self.GetParent().KeySelect(self.GetParent().keytree.GetSelection())
		except:
			pass
		#
		selected_fields=[]
		for i in self.searchFields.GetSelections():
			selected_fields.append(self.searchFields.GetString(i))
		BIB.BIB_SEARCH_FIELDS = tuple(selected_fields)
		#
		BIB.KEY_COLOR = self.colors[self.key_color.GetStringSelection()]
		BIB.TEXT_COLOR = self.colors[self.text_color.GetStringSelection()]
		#
		# we keep a copy of the current settings to know if we must re-connect onOK
		saved_db = BIB.DB_STARTUP,BIB.DB_TYPE,BIB.STORE_PASSWD,BIB.DB_NAME,BIB.USER,BIB.PASSWORD,BIB.HOST,BIB.PORT,BIB.SOCKET,BIB.SQLiteFile,BIB.SQLiteUSER
		BIB.DB_STARTUP,BIB.DB_TYPE,BIB.STORE_PASSWD,DB_NAME,USER,PASSWORD,HOST,PORT,SOCKET,SQLiteFile,SQLiteUSER = self.prefDB.getSettings()
		#
		if BIB.DB_STARTUP == 1:					# db from default
			if BIB.DB_TYPE == 'MySQL':
				BIB.DB_NAME = DB_NAME
				BIB.USER = USER
				if BIB.STORE_PASSWD:
					BIB.PASSWORD = PASSWORD
				else:
					BIB.PASSWORD = ''
				BIB.HOST = HOST
				BIB.PORT = PORT
				BIB.SOCKET = SOCKET
			elif BIB.DB_TYPE == 'SQLite':
				BIB.SQLiteFile = SQLiteFile
				BIB.SQLiteUSER = SQLiteUSER
		#
		# saving duplicates settings
		BIB.DUPLICATES_TEST, BIB.DUPLICATES, BIB.DUPLICATES_CASE, BIB.DUPLICATES_KEEP_OLD = self.prefDuplicates.getSettings()
		# saving printing settings
		BIB.PRINTER_FORMAT, BIB.PRINTER_COLORS, BIB.PRINTER_STYLE, BIB.PRINTER_USE_OOo_FORMAT = self.prefPrinting.getSettings()
		# saving Reference editor settings
		BIB.EDIT = self.prefEditor.getSettings()
		# saving Shortcuts settings
		BIB.SHORTCUTS, BIB.SHORTFILE, BIB.SHORT = self.prefShortcuts.getSettings()
		# saving wp connection settings
		BIB.WP,BIB.OO_CON_TYPE,BIB.OO_PIPE,BIB.OO_HOST,BIB.OO_PORT = self.prefConnection.getSettings()
		self.GetParent().switchConnectionMenu()			# we switch the menu from OOo/word/nothing if needed
		# saving paths settings
		BIB.FILES = self.prefPaths.getSettings()
		#
		BIB.CONFIG.writeConfig(bool(BIB.DB_STARTUP))	# save the config changes with or without the database
		#
		# loading dbBib module corresponding to the database type choice. if needed
		if saved_db != (BIB.DB_STARTUP,BIB.DB_TYPE,BIB.STORE_PASSWD,BIB.DB_NAME,BIB.USER,BIB.PASSWORD,BIB.HOST,BIB.PORT,BIB.SOCKET,BIB.SQLiteFile,BIB.SQLiteUSER):
			try:
				BIB.DB_MODULE = __import__('dbBib'+BIB.DB_TYPE)
				getattr(BIB.DB_MODULE,'dbBib')	# if the module was loaded before, re-importing won't raise ImportError. We must check directly if it works
			except (ImportError,AttributeError):
				BIB.DB_MODULE = None
				wxLogError(_("""Sorry, but I was not able to find the python module for the %s database.\nPlease check your installation""")%BIB.DB_TYPE)
			self.GetParent().autoConnect()		# connect to the new database
			#
		self.Close()

	def onCloseWindow(self,event):
		self.Destroy()

	def add(self,event):
		if self.fields.GetSelection() != -1:
			self.fieldsChoice.Append(self.fields.GetStringSelection())
			self.fields.Delete(self.fields.GetSelection())
	def remove(self,event):
		if self.fieldsChoice.GetSelection() != -1:
			self.fields.Append(self.fieldsChoice.GetStringSelection())
			self.fieldsChoice.Delete(self.fieldsChoice.GetSelection())
	def up(self,event):
		selId = self.fieldsChoice.GetSelection()
		if selId > 0:	# it is not the first item
			selStr = self.fieldsChoice.GetStringSelection()
			self.fieldsChoice.Delete(selId)
			self.fieldsChoice.InsertItems([selStr],selId-1)
			self.fieldsChoice.SetSelection(selId-1,True)
	def down(self,event):
		selId = self.fieldsChoice.GetSelection()
		if selId+1 < self.fieldsChoice.GetCount() and selId != -1:	# not the last item
			selStr = self.fieldsChoice.GetStringSelection()
			self.fieldsChoice.Delete(selId)
			self.fieldsChoice.InsertItems([selStr],selId+1)
			self.fieldsChoice.SetSelection(selId+1,True)

