# Copyright 2004,2005 Pierre Martineau <pmartino@users.sourceforge.net>
# This file is part of Bibus, a bibliographic database that can
# work together with OpenOffice.org to generate bibliographic indexes.
#
# Bibus is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bibus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bibus; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import wx, BIB, DnD, webbrowser, os

class RefList(wx.ListCtrl):
	def __init__(self,parent,bibus_frame,style=wx.LC_REPORT|wx.LC_VIRTUAL|wx.LC_HRULES|wx.LC_VRULES,refs=()):
		"""refs = list/tuple containing the references to display
		The first column = ItemData = Id in database
		bibus_frame = main window Frame
		refs[item][0] = id in db
		refs[item][1:] = values to display
		"""
		self.bibus_frame = bibus_frame
		self.refs = ()
		self.datas = ()
		self.lastsort = None		# to remember the last sorted column and the order: (n col, 'up') or (n, 'dn' )
		wx.ListCtrl.__init__(self,parent,-1,style=style)
		self.resizeList()
		self.__set_evt()

	def setList(self,refs):
		self.DeleteAllItems()
		self.SetItemCount( len(refs) )
		self.refs = refs
		self.datas = [ref[0] for ref in refs]

	def OnGetItemText(self,item,column):
		if item < len(self.refs):
			return apply(BIB.LIST_FIELD_FORMAT[BIB.LIST_DISPLAY[column]],(self.refs[item][column+1],))

	def GetItemData(self,item):
		return self.datas[item]

	def FindItemData(self,start,data):
		if start == -1:
			return self.datas.index(data)
		else:
			return self.datas.index(data,start)

	def noSort(self):
		self.lastsort = None	# we clear the last ordering of reflist

	def resizeList(self):
		"""Insert Column names and resizes"""
		self.ClearAll()
		for i in xrange(len(BIB.LIST_DISPLAY)):
			self.InsertColumn(i,BIB.LIST_DISPLAY[i])
			try:
				self.SetColumnWidth(i, BIB.LIST_COL_SIZE[i])
			except IndexError:
				self.SetColumnWidth(i, BIB.LIST_COL_SIZE_DEFAULT)

	def __set_evt(self):
		wx.EVT_LIST_COL_CLICK(self,self.GetId(),self.onSortList)			# sort list in ascending order of the clicked column
		wx.EVT_LIST_COL_END_DRAG(self,self.GetId(),self.onDragColumn)
		wx.EVT_LIST_ITEM_DESELECTED(self,self.GetId(),self.onDeselectRef)
		wx.EVT_LIST_ITEM_SELECTED(self,self.GetId(),self.onSelectRef)
		wx.EVT_LIST_BEGIN_DRAG(self,self.GetId(),self.onDragRef)
		wx.EVT_LIST_ITEM_ACTIVATED(self,self.GetId(),self.onEditRef)		# Double click OR enter
		wx.EVT_LIST_ITEM_RIGHT_CLICK(self,self.GetId(),self.onListRightClick)
		wx.EVT_LIST_ITEM_MIDDLE_CLICK(self,self.GetId(),self.onOpenURL)	# doesn't work with the openoffice python 2.2.2 under linux!
		#wx.EVT_LIST_KEY_DOWN(self,self.GetId(),self.onKeyDown)
		wx.EVT_CHAR(self,self.onKeyDown)
		# PopUp menu ListCtrl
		[self.popTagRef,self.popEditRef, self.popNewRef, self.popDeleteRef, self.popOpenURL] = [wx.NewId() for x in range(5)]
		wx.EVT_MENU(self,self.popTagRef,self.bibus_frame.onMenuTagRef)
		wx.EVT_MENU(self,self.popEditRef,self.bibus_frame.onMenuEditRef)
		wx.EVT_MENU(self,self.popNewRef,self.bibus_frame.onMenuNewRef)
		wx.EVT_MENU(self,self.popDeleteRef,self.bibus_frame.onMenuDelRef)
		wx.EVT_MENU(self,self.popOpenURL,self.onMenuOpenURL)

	def onSortList(self,event):
		"""Sort the list in the ascending order of the clicked column"""
		if event.GetColumn()!= -1:
			if self.lastsort == (event.GetColumn() , 'up'):	# this column was the last sorted => we now sort by DESCending order
				self.bibus_frame.KeySelect( self.bibus_frame.keytree.GetSelection(),order=BIB.LIST_DISPLAY[event.GetColumn()] ,how='DESC')
				self.lastsort = (event.GetColumn() , 'dn')
				self.SetColumnImage(event.GetColumn(),BIB.ARROW_DN)
			else:											# we sort by ASCending order
				self.bibus_frame.KeySelect( self.bibus_frame.keytree.GetSelection(),order=BIB.LIST_DISPLAY[event.GetColumn()] ,how='ASC')
				self.lastsort = (event.GetColumn() , 'up')
				self.SetColumnImage(event.GetColumn(),BIB.ARROW_UP)

	def onDragColumn(self,event):
		"""Resizing column"""
		tmp = []
		for i in xrange( self.GetColumnCount() ):
			tmp.append( self.GetColumnWidth(i) )
		BIB.LIST_COL_SIZE = tuple(tmp)

	def onDeselectRef(self,event):
		self.bibus_frame.bibframe1_statusbar.SetStatusText(_("%s references : %s selected")%(self.GetItemCount(),self.GetSelectedItemCount()),1)
		self.bibus_frame.set_menu(self.bibus_frame.keytree.GetSelection())	# update menu

	def onSelectRef(self,event):
		""" Display the full selected reference. If multiple selection -> no display"""
		self.bibus_frame.bibframe1_statusbar.SetStatusText(_("%s references : %s selected")%(self.GetItemCount(),self.GetSelectedItemCount()),1)
		self.bibus_frame.set_menu(self.bibus_frame.keytree.GetSelection())	# update menu
		if self.GetSelectedItemCount() == 1:
			ref_id = self.GetItemData(self.GetFirstSelected())
			ref = self.bibus_frame.db.getRef(ref_id,BIB.BIB_FIELDS)
			id = self.bibus_frame.keytree.GetPyData(self.bibus_frame.keytree.GetSelection())[2]
			if id == BIB.ID_IMPORT_ROOT:	# we are in the import list
				ref = self.bibus_frame.db.getRefImport(ref_id,BIB.BIB_FIELDS)
			elif id == BIB.ID_ONLINE_ROOT:	# we are in the online list
				ref = self.bibus_frame.db.getRefOnline(ref_id,BIB.BIB_FIELDS)
			else:							# we are in ref or query
				ref = self.bibus_frame.db.getRef(ref_id,BIB.BIB_FIELDS)
			# we display the reference
			self.bibus_frame.refDisplaypanel.display(ref[0])

	def onDragRef(self,event):
		"Drag list of Id of the selected references"
		item = self.bibus_frame.keytree.GetSelection()
		oldkey_id,user,id = self.bibus_frame.keytree.GetPyData(item)
		allowed=BIB.ALLOWED[id]
		if allowed & BIB.REF_DRAG_OK:
			if id == BIB.ID_ONLINE_ROOT or id == BIB.ID_ONLINE:
				key = u'Online'
			elif id == BIB.ID_IMPORT_ROOT or id == BIB.ID_IMPORT:
				key = u'Import'
			else:
				key = u''
			#
			dragData = DnD.bibDragData(self.bibus_frame.db.getDbInfo(key),BIB.DnD_REF)
			item = self.GetFirstSelected()
			if id == BIB.ID_IMPORT_ROOT:	# we are in the import list
				f = self.bibus_frame.db.getRefImport
			elif id == BIB.ID_ONLINE_ROOT:	# we are in the online list
				f = self.bibus_frame.db.getRefOnline
			else:				# we are in ref or query
				f = self.bibus_frame.db.getRef
			while item != -1:
				dragData.append(tuple(apply(f,(self.GetItemData(item),BIB.BIB_FIELDS))[0]))
				# tuple is needed for sqlite since returned tuple is of class sqlite.main.PgResultSetConcreteClass
				item = self.GetNextSelected(item)
			mydrag = DnD.bibDataObject()
			mydrag.SetData(dragData)
			dragsource = wx.DropSource(self)
			dragsource.SetData(mydrag)
			result = dragsource.DoDragDrop(wx.Drag_DefaultMove)
			if allowed & BIB.REF_DEL_OK and result == wx.DragMove:
				for ref in dragData:
					self.bibus_frame.db.delLink(oldkey_id,ref[0])
				self.bibus_frame.updateList(self.bibus_frame.db.getRefKey(oldkey_id))
		else:
			event.Veto()
			self.showError(_("Sorry, you cannot move a reference associated with this key."))

	def onEditRef(self,event):
		self.bibus_frame.onMenuEditRef(None)

	def onListRightClick(self,event):
		where = event.GetPoint()	# position relative to the window
		pw = self.GetPosition()
		where.x = where.x + pw.x
		where.y = where.y + pw.y
		# deselect everything if we did not click in a selected item
		if not self.IsSelected(event.GetIndex()):
			item = self.GetFirstSelected()
			while item != -1:
				self.Select(item,False)
				item = self.GetNextSelected(item)
			self.Select(event.GetIndex(),True)	# select the item
		self.Focus(event.GetIndex())			# focus
		key_id,user,id = self.bibus_frame.keytree.GetPyData(self.bibus_frame.keytree.GetSelection())
		popUp = wx.Menu()
		if BIB.ALLOWED[id] & BIB.REF_TAG_OK:
			popUp.Append(self.popTagRef,_('Tag'))
		if BIB.ALLOWED[id] & BIB.REF_EDIT_OK:
			if popUp.GetMenuItemCount() > 0: popUp.AppendSeparator()
			popUp.Append(self.popEditRef,_('Edit'))
		if BIB.ALLOWED[id] & BIB.REF_NEW_OK:
			popUp.Append(self.popNewRef,_('New'))
		if BIB.ALLOWED[id] & BIB.REF_DEL_OK:
			popUp.Append(self.popDeleteRef,_('Delete'))
		if popUp.GetMenuItemCount() > 0:
			popUp.AppendSeparator()
		popUp.Append(self.popOpenURL,_('Open URL'))
		if popUp.GetMenuItemCount() > 0:
			self.PopupMenu(popUp,where)

	def __openInBrowser(self,ref_id):
		"""Open the ref ref_id in a webbbrowser if there is a non empty URL field"""
		key_id = self.bibus_frame.keytree.GetPyData(self.bibus_frame.keytree.GetSelection())[2]
		if key_id in (BIB.ID_REF_ROOT,BIB.ID_REF,BIB.ID_REF_ALL,BIB.ID_QUERY,BIB.ID_CITED):
			url = self.bibus_frame.db.getRef(ref_id,('URL',))[0][0]
		elif key_id in (BIB.ID_ONLINE,BIB.ID_ONLINE_ROOT):
			url = self.bibus_frame.db.getRefOnline(ref_id,('URL',))[0][0]
		elif key_id in (BIB.ID_IMPORT,BIB.ID_IMPORT_ROOT):
			url = self.bibus_frame.db.getRefImport(ref_id,('URL',))[0][0]
		else:
			url = ''
		if url:
			# we first try to open using startfile which is available only under windows, then webbrowser
			# we use expandvars to replace $PDF with the name of the directory containing the PDF files
			try:
				os.startfile(os.path.expandvars(url))
			except:
				webbrowser.open(os.path.expandvars(url),new=1)

	def onMenuOpenURL(self,event):	# popUp menu
		"""Open the selected refs in a WebBrowser when menu selected"""
		item = self.GetFirstSelected()
		while item != -1:
			self.__openInBrowser(self.GetItemData(item))
			item = self.GetNextSelected(item)

	def onOpenURL(self,event):		# middle click
		"""Open the selected refs in a WebBrowser when middle click in it"""
		# deselect everything if we did not click in a selected item
		if not self.IsSelected(event.GetIndex()):
			item = self.GetFirstSelected()
			while item != -1:
				self.Select(item,False)
				item = self.GetNextSelected(item)
			self.Select(event.GetIndex())	# select the item
		self.Focus(event.GetIndex())	# focus on the clicked reference
		self.onMenuOpenURL(event)

	def onKeyDown(self,event):
		id = self.bibus_frame.keytree.GetPyData(self.bibus_frame.keytree.GetSelection())[2]
		if (event.GetKeyCode() == wx.WXK_DELETE) and (BIB.ALLOWED[id] & BIB.REF_DEL_OK):
			self.bibus_frame.onMenuDelRef(None)
		elif ( event.GetKeyCode() == ord(BIB.KEY_TAGGED) ) and (BIB.ALLOWED[id] & BIB.REF_TAG_OK):
			self.bibus_frame.onMenuTagRef(None)
		elif ( event.GetKeyCode() == ord(BIB.KEY_SWITCH_DISPLAY) ):
			self.bibus_frame.refDisplaypanel.switchDisplay()
			self.SetFocus()
		else:
			event.Skip()
