# Copyright 2004,2005 Pierre Martineau <pmartino@users.sourceforge.net>
# This file is part of Bibus, a bibliographic database that can
# work together with OpenOffice.org to generate bibliographic indexes.
#
# Bibus is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bibus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bibus; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
from wxPython.wx import *
#wxToolBar,wxChoice,wxTextCtrl,wxButton,EVT_TEXT_ENTER,wxTE_PROCESS_ENTER,wxTE_LEFT,EVT_BUTTON,wxSIZE_USE_EXISTING
import Search,SearchMedline

class myToolBar(wxPanel):
	def __init__(self,*args, **kwds):
		wxPanel.__init__(self,*args, **kwds)
		self.parent=self.GetParent()
		self.choice1 = wxChoice(self,-1,choices=[_("Search"),_("Expert Search"),_("Search PubMed")])
		self.choice1.SetSelection(0)
		self.searchtxt1 = wxTextCtrl(self,-1,style=wxTE_PROCESS_ENTER|wxTE_LEFT)
		self.showall = wxButton(self,-1,_("Show All"))
		self.showall.Enable(False)
		#
		s0 = wxBoxSizer(wxHORIZONTAL)
		s0.Add(self.choice1,0,wxEXPAND)
		s0.Add(self.searchtxt1,1,wxALIGN_CENTER_VERTICAL)
		s0.Add(self.showall,0,wxALIGN_CENTER_VERTICAL)
		#
		self.SetSizerAndFit(s0)
		self.Layout()
		self.__set_evt()

	def __set_evt(self):
		EVT_TEXT_ENTER(self,self.searchtxt1.GetId(),self.onSearch)
		EVT_BUTTON(self,self.showall.GetId(),self.onShowAll)

	def __search(self,event,searchstr):
		search = Search.Search(self.parent,self.parent,-1,u'')
		search.SearchValue[0].SetValue(searchstr)
		search.onSearchClose(event)
		self.showall.Enable(True)

	def __searchPubMed(self,event,searchstr):
		search = SearchMedline.SearchMedline(self.parent.db,self.parent,-1,u'')
		search.SearchValue[0].SetValue(searchstr)
		search.onSearchClose(event)

	def __query(self,event,searchstr):
		search = Search.Search(self.parent,self.parent,-1,u'')
		search.Mode.SetStringSelection(_('Expert'))
		search.ExpertValue.SetValue(searchstr)
		search.onSearchClose(event)
		self.showall.Enable(True)

	def onSearch(self,event):
		#print event.GetSelection()
		if self.choice1.GetSelection() ==  0:	# choice search
			self.__search(event,self.searchtxt1.GetValue())
		elif self.choice1.GetSelection() ==  2:	# choice PubMed
			self.__searchPubMed(event,self.searchtxt1.GetValue())
		else:					# choice query = expert mode
			self.__query(event,self.searchtxt1.GetValue())

	def displaySearch(self,searchstr):
		# put searchstr in the search field and enable showall button
		self.searchtxt1.SetValue(searchstr)
		self.showall.Enable(True)

	def GetClientWidth(self):
		if wxPlatform == '__WXGTK__':	# bug in wxPython gtk
			return self.GetSize().GetWidth()-10
		else:
			return self.GetClientSize().GetWidth()

	def onShowAll(self,event):
		self.clearSearch()
		self.onSearch(event)
		self.showall.Enable(False)

	def clearSearch(self):
		self.searchtxt1.SetValue(u'')
		self.showall.Enable(False)

	def withoutPubMed(self):
		"""Remove the search PubMed choice"""
		if self.choice1.GetCount() == 3:
			self.choice1.Delete(2)
			self.choice1.SetSelection(self.choice1.GetSelection())	# needed to reset the choice under GTK

	def withPubMed(self):
		"""Add the search PubMed choice"""
		if self.choice1.GetCount() == 2:
			self.choice1.Append( _("Search PubMed") )

